package com.sillycycle.bagleyd.welltris;

/*
 * @(#)PolyominoImage.java
 *
 * createImages() originally written in C
 * Copyright 2000  Stephen Montgomery-Smith
 *
 * Copyright 1993 - 2018  David A. Bagley, bagleyd AT verizon.net
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995 Q. Alex Zhao, azhao AT cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;

import com.sillycycle.bagleyd.util.PatternColorFilter;

/**
 * The <code>PolyominoImage</code> class holds the colors and
 * handles the construction of the images for the different
 * polyominoes.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/altris.html
 */

public class PolyominoImage extends Component {
	private static final long serialVersionUID = 42L;
	boolean plainPiece = false;
	private static final boolean SMALL_BELLYBUTTON = false;
	private static final boolean LARGE_BELLYBUTTON = false;
	private static final boolean BG_DARK = true;

	static String[] colorStrings = { // 71
		"Red", "Green", "Blue", "Yellow",
		"Magenta", "Cyan", "FireBrick", "GreenYellow",
		"SlateBlue", "Khaki", "Plum", "Violet",
		"DarkTurquoise", "Gold", "Orchid", "Turquoise",
		"Orange", "OrangeRed", "VioletRed", "BlueViolet",
		"SeaGreen", "Pink", "ForestGreen", "SkyBlue",
		"Coral", "Wheat", "Goldenrod", "IndianRed",
		"SpringGreen", "CornflowerBlue", "Thistle", "Aquamarine",
		"CadetBlue", "LightSteelBlue", "NavyBlue", "SteelBlue",
		"YellowGreen", "DarkViolet", "MediumSeaGreen", "DarkSlateGray",
		"White", "MediumVioletRed", "Sienna", "MediumAquamarine",
		"MediumBlue", "Navy", "DarkOliveGreen", "DarkGreen",
		"DimGray", "Tan", "MediumTurquoise", "DarkSlateBlue",
		"Maroon", "Gray", "Black"
	};

	static Color[] colorThings = {
		Color.red, Color.green, Color.blue, Color.yellow,
		Color.magenta, Color.cyan,
			new Color(178, 34, 34), new Color(173, 255, 47),
		new Color(131, 111, 255), new Color(255, 246, 143),
			new Color(255, 187, 255), new Color(238, 130, 238),
		new Color(0, 206, 209), new Color(255, 215, 0),
			new Color(218, 112, 214), new Color(0, 229, 238),
		new Color(255, 165, 0), new Color(255, 69, 0),
			new Color(208, 32, 144), new Color(138, 43, 226),
		new Color(78, 238, 148), new Color(238, 169, 184),
			new Color(34, 139, 34), new Color(126, 192, 238),
		new Color(255, 127, 80), new Color(245, 222, 179),
			new Color(218, 165, 32), new Color(205, 92, 92),
		new Color(0, 238, 118), new Color(100, 149, 237),
			new Color(216, 191, 216), new Color(127, 255, 212),
		new Color(95, 158, 160), new Color(176, 196, 222),
			new Color(0, 0, 128), new Color(70, 130, 180),
		new Color(154, 205, 50), new Color(148, 0, 211),
			new Color(60, 179, 113), new Color(151, 255, 255),
		Color.white, new Color(199, 21, 133),
			new Color(238, 121, 66), new Color(102, 205, 170),
		new Color(0, 0, 208), new Color(0, 0, 177),
			new Color(85, 107, 47), new Color(0, 100, 0),
		new Color(170, 170, 170), new Color(210, 180, 140),
			new Color(72, 209, 204), new Color(72, 61, 139),
		new Color(176, 48, 96), Color.gray, Color.black
	};

	public static final int BIG = 256;
	public static final int SMALL = 16;
	public static final int NUM_BITMAPS = BIG;
	static final int LEFT = 1; /* (1<<0) */
	static final int DOWN = 2; /* (1<<1) */
	static final int RIGHT = 4; /* (1<<2) */
	static final int UP = 8; /* (1<<3) */
	static final int LEFT_DOWN = 16; /* (1<<4) */
	static final int RIGHT_DOWN = 32; /* (1<<5) */
	static final int RIGHT_UP = 64; /* (1<<6) */
	static final int LEFT_UP = 128; /* (1<<7) */
	static final int DRAW_COLOR = 0;
	static final int XOR_COLOR = -1;
	static final int XOR_CLEAR_COLOR = -2;
	static final int CLEAR_COLOR = -3;
	StringBuilder[] data = null;
	private static final char[] PATTERN_CHARS = {'*', '.'};

	static boolean IS_LEFT(int n) {
		return ((n & LEFT) != 0);
	}

	static boolean IS_DOWN(int n) {
		return ((n & DOWN) != 0);
	}

	static boolean IS_RIGHT(int n) {
		return ((n & RIGHT) != 0);
	}

	static boolean IS_UP(int n) {
		return ((n & UP) != 0);
	}

	static boolean IS_LEFT_DOWN(int n) {
		return ((n & LEFT_DOWN) != 0);
	}

	static boolean IS_RIGHT_DOWN(int n) {
		return ((n & RIGHT_DOWN) != 0);
	}

	static boolean IS_RIGHT_UP(int n) {
		return ((n & RIGHT_UP) != 0);
	}

	static boolean IS_LEFT_UP(int n) {
		return ((n & LEFT_UP) != 0);
	}

	public static int CHECKLEFT(int x) {
		return (x | LEFT);
	}

	public static int CHECKDOWN(int x) {
		return (x | DOWN);
	}

	public static int CHECKRIGHT(int x) {
		return (x | RIGHT);
	}

	public static int CHECKUP(int x) {
		return (x | UP);
	}

	/* Defines to access the bitmaps. */

	void SETBIT(int x, int y) {
		data[y].setCharAt(x, PATTERN_CHARS[1]);
	}

	void RESBIT(int x, int y) {
		data[y].setCharAt(x, PATTERN_CHARS[0]);
	}

	void HALFBIT(int x, int y) {
		if ((x + y) % 2 == 0)
			RESBIT(x, y);
		else
			SETBIT(x, y);
	}

	void NOTHALFBIT(int x, int y) {
		if ((x + y) % 2 == 0)
			SETBIT(x, y);
		else
			RESBIT(x, y);
	}

	void THIRDBIT(int x, int y) {
		if ((x + y + 2) % 3 == 0)
			SETBIT(x, y);
		else
			RESBIT(x, y);
	}

	void TWOTHIRDSBIT(int x, int y) {
		if ((x + y + 2) % 3 == 0)
			RESBIT(x, y);
		else
			SETBIT(x, y);
	}

	void THREEQUARTERSBIT(int x, int y) {
		if (((y & 1) == 1) || (((x + 2 + y / 2 + 1) & 1) == 1))
			SETBIT(x, y);
		else
			RESBIT(x, y);
	}

	/* Parameters for bitmaps. */
	static int G(int boxsize) { // 1/2 of gap between pentominoes.
		return (boxsize / 45) + 1;
	}

	static int T(int boxsize) { // Thickness of walls of pentominoes.
		return ((boxsize <= 12) ? 1 : G(boxsize * 2));
	}

	static int R(int boxsize) { // Amount of rounding.
		return ((boxsize <= 12) ? 1 : G(boxsize * 6));
	}

	static int RT(int boxsize) { // Thickness of wall of rounded parts.
			// Here 3 is an approximation to 2 sqrt(2).
		return ((boxsize <= 12) ? 1 : G(boxsize * 3));
	}

	static int RR() { // Roof ridge thickness
		return 0;
	}

	/* A list of those bitmaps we need to create to display any polyomino. */
	static int[] imagesNeeded = {
		// This needed for monomino
		LEFT | RIGHT | UP | DOWN | LEFT_UP | LEFT_DOWN | RIGHT_UP | RIGHT_DOWN,

		LEFT_UP | LEFT_DOWN | RIGHT_UP | RIGHT_DOWN,

		LEFT | RIGHT_UP | RIGHT_DOWN,
		RIGHT | LEFT_UP | LEFT_DOWN,
		UP | LEFT_DOWN | RIGHT_DOWN,
		DOWN | LEFT_UP | RIGHT_UP,
		LEFT | RIGHT_UP,
		RIGHT | LEFT_UP,
		UP | LEFT_DOWN,
		DOWN | LEFT_UP,
		LEFT | RIGHT_DOWN,
		RIGHT | LEFT_DOWN,
		UP | RIGHT_DOWN,
		DOWN | RIGHT_UP,

		// These needed for hexominoes
		LEFT,
		RIGHT,
		UP,
		DOWN,
		LEFT_DOWN | RIGHT_UP | RIGHT_DOWN,
		LEFT_UP | RIGHT_UP | RIGHT_DOWN,
		LEFT_UP | LEFT_DOWN | RIGHT_DOWN,
		LEFT_UP | LEFT_DOWN | RIGHT_UP,

		LEFT | UP | RIGHT_DOWN,
		LEFT | DOWN | RIGHT_UP,
		RIGHT | UP | LEFT_DOWN,
		RIGHT | DOWN | LEFT_UP,
		LEFT | UP,
		LEFT | DOWN,
		RIGHT | UP,
		RIGHT | DOWN,

		LEFT | RIGHT,
		UP | DOWN,

		RIGHT | UP | DOWN,
		LEFT | UP | DOWN,
		LEFT | RIGHT | DOWN,
		LEFT | RIGHT | UP,

		-1
	};

	static Image[][][] pms = new Image[2][2][NUM_BITMAPS];

	static boolean imageNeeded(int n) {
		for (int i = 0; imagesNeeded[i] != -1; i++)
			if (n == imagesNeeded[i])
				return true;
		return false;
	}

	Image createBitmapPiece(int n, boolean use3D, int boxsize,
			Color background) {
		Color[] patternColors = {background, new Color(0, true)};
		int x, y; // unsigned
		char[] empty = new char[boxsize];

		for (x = 0; x < boxsize; x++)
			empty[x] = '.';
		data = new StringBuilder[boxsize];

		for (y = 0; y < boxsize; y++)
			data[y] = new StringBuilder(new String(empty));
		if (data == null) {
			System.out.println("No images");
			return null; // no biggie, will not use images
		}
		for (y = 0; y < boxsize; y++)
		  for (x = 0; x < boxsize; x++) {
		    if (!use3D) {
		      if (SMALL_BELLYBUTTON) {
			if (x >= boxsize / 2 && x <= boxsize / 2 + 1 &&
			    y >= boxsize / 2 && y <= boxsize / 2 + 1)
				NOTHALFBIT(x, y);
			else
				HALFBIT(x, y);
		      } else
			HALFBIT(x, y);
		    } else if ((x >= y && x <= boxsize - y - 1 && IS_UP(n)) ||
			(x <= y && x <= boxsize - y - 1 && y < boxsize / 2 &&
			  !IS_LEFT(n)) ||
			(x >= y && x >= boxsize - y - 1 && y < boxsize / 2 &&
			  !IS_RIGHT(n))) {
			if (BG_DARK)
				SETBIT(x, y);
			else
				THIRDBIT(x, y);
		    } else if ((x <= y && x <= boxsize - y - 1 && IS_LEFT(n)) ||
			(x >= y && x <= boxsize - y - 1 && x < boxsize / 2 &&
			  !IS_UP(n)) ||
			(x <= y && x >= boxsize - y - 1 && x < boxsize / 2 &&
			  !IS_DOWN(n))) {
			if (BG_DARK)
				TWOTHIRDSBIT(x, y);
			else
				HALFBIT(x, y);
		    } else if ((x >= y && x >= boxsize - y - 1 && IS_RIGHT(n)) ||
			(x >= y && x <= boxsize - y - 1 && x >= boxsize / 2 &&
			  !IS_UP(n)) ||
			(x <= y && x >= boxsize - y - 1 && x >= boxsize / 2 &&
			  !IS_DOWN(n))) {
			if (BG_DARK)
				HALFBIT(x, y);
			else
				TWOTHIRDSBIT(x, y);
		    } else if ((x <= y && x >= boxsize - y - 1 && IS_DOWN(n)) ||
			(x <= y && x <= boxsize - y - 1 && y >= boxsize / 2 &&
			  !IS_LEFT(n)) ||
			(x >= y && x >= boxsize - y - 1 && y >= boxsize / 2 &&
			  !IS_RIGHT(n))) {
			if (BG_DARK)
				THIRDBIT(x, y);
			else
				SETBIT(x, y);
		    }
		  }

		if (IS_LEFT(n))
		  for (y=0;y<boxsize;y++)
		    for (x=G(boxsize);x<G(boxsize)+T(boxsize);x++)
		      SETBIT(x, y);
		if (IS_RIGHT(n))
		  for (y=0;y<boxsize;y++)
		    for (x=G(boxsize);x<G(boxsize)+T(boxsize);x++)
		      SETBIT(boxsize-1-x, y);
		if (IS_UP(n))
		  for (x=0;x<boxsize;x++)
		    for (y=G(boxsize);y<G(boxsize)+T(boxsize);y++)
		      SETBIT(x, y);
		if (IS_DOWN(n))
		  for (x=0;x<boxsize;x++)
		    for (y=G(boxsize);y<G(boxsize)+T(boxsize);y++)
		      SETBIT(x,boxsize-1-y);
		if (IS_LEFT(n))
		  for (y=0;y<boxsize;y++)
		    for (x=0;x<G(boxsize);x++)
		      RESBIT(x, y);
		if (IS_RIGHT(n))
		  for (y=0;y<boxsize;y++)
		    for (x=0;x<G(boxsize);x++)
		      RESBIT(boxsize-1-x, y);
		if (IS_UP(n))
		  for (x=0;x<boxsize;x++)
		    for (y=0;y<G(boxsize);y++)
		      RESBIT(x, y);
		if (IS_DOWN(n))
		  for (x=0;x<boxsize;x++)
		    for (y=0;y<G(boxsize);y++)
		      RESBIT(x,boxsize-1-y);

		if (IS_LEFT(n) && IS_UP(n))
		  for (x=G(boxsize);x<=G(boxsize)+R(boxsize);x++)
		    for (y=G(boxsize);y<=R(boxsize)+2*G(boxsize)-x;y++) {
		      if (x+y>R(boxsize)+2*G(boxsize)-RT(boxsize))
			SETBIT(x, y);
		      else
			RESBIT(x, y);
		    }
		if (IS_LEFT(n) && IS_DOWN(n))
		  for (x=G(boxsize);x<=G(boxsize)+R(boxsize);x++)
		    for (y=G(boxsize);y<=R(boxsize)+2*G(boxsize)-x;y++) {
		      if (x+y>R(boxsize)+2*G(boxsize)-RT(boxsize))
			SETBIT(x,boxsize-1-y);
		      else
			RESBIT(x,boxsize-1-y);
		    }
		if (IS_RIGHT(n) && IS_UP(n))
		  for (x=G(boxsize);x<=G(boxsize)+R(boxsize);x++)
		    for (y=G(boxsize);y<=R(boxsize)+2*G(boxsize)-x;y++) {
		      if (x+y>R(boxsize)+2*G(boxsize)-RT(boxsize))
			SETBIT(boxsize-1-x, y);
		      else
			RESBIT(boxsize-1-x, y);
		    }
		if (IS_RIGHT(n) && IS_DOWN(n))
		  for (x=G(boxsize);x<=G(boxsize)+R(boxsize);x++)
		    for (y=G(boxsize);y<=R(boxsize)+2*G(boxsize)-x;y++) {
		      if (x+y>R(boxsize)+2*G(boxsize)-RT(boxsize))
			SETBIT(boxsize-1-x,boxsize-1-y);
		      else
			RESBIT(boxsize-1-x,boxsize-1-y);
		    }

		if (!IS_LEFT(n) && !IS_UP(n) && IS_LEFT_UP(n)) {
		  for (x=0;x<G(boxsize);x++)
		    for (y=0;y<G(boxsize);y++)
		      RESBIT(x, y);
		  for (x=G(boxsize);x<G(boxsize)+T(boxsize);x++)
		    for (y=0;y<G(boxsize);y++)
		      SETBIT(x, y);
		  for (x=0;x<G(boxsize)+T(boxsize);x++)
		    for (y=G(boxsize);y<G(boxsize)+T(boxsize);y++)
		      SETBIT(x, y);
		}
		if (!IS_LEFT(n) && !IS_DOWN(n) && IS_LEFT_DOWN(n)) {
		  for (x=0;x<G(boxsize);x++)
		    for (y=0;y<G(boxsize);y++)
		      RESBIT(x,boxsize-1-y);
		  for (x=G(boxsize);x<G(boxsize)+T(boxsize);x++)
		    for (y=0;y<G(boxsize);y++)
		      SETBIT(x,boxsize-1-y);
		  for (x=0;x<G(boxsize)+T(boxsize);x++)
		    for (y=G(boxsize);y<G(boxsize)+T(boxsize);y++)
		      SETBIT(x,boxsize-1-y);
		}
		if (!IS_RIGHT(n) && !IS_UP(n) && IS_RIGHT_UP(n)) {
		  for (x=0;x<G(boxsize);x++)
		    for (y=0;y<G(boxsize);y++)
		      RESBIT(boxsize-1-x, y);
		  for (x=G(boxsize);x<G(boxsize)+T(boxsize);x++)
		    for (y=0;y<G(boxsize);y++)
		      SETBIT(boxsize-1-x, y);
		  for (x=0;x<G(boxsize)+T(boxsize);x++)
		    for (y=G(boxsize);y<G(boxsize)+T(boxsize);y++)
		      SETBIT(boxsize-1-x, y);
		}
		if (!IS_RIGHT(n) && !IS_DOWN(n) && IS_RIGHT_DOWN(n)) {
		  for (x=0;x<G(boxsize);x++)
		    for (y=0;y<G(boxsize);y++)
		      RESBIT(boxsize-1-x,boxsize-1-y);
		  for (x=G(boxsize);x<G(boxsize)+T(boxsize);x++)
		    for (y=0;y<G(boxsize);y++)
		      SETBIT(boxsize-1-x,boxsize-1-y);
		  for (x=0;x<G(boxsize)+T(boxsize);x++)
		    for (y=G(boxsize);y<G(boxsize)+T(boxsize);y++)
		      SETBIT(boxsize-1-x,boxsize-1-y);
		}

		if (use3D) {
			if (NUM_BITMAPS != SMALL) {
				if (!IS_LEFT(n) && !IS_UP(n) && !IS_LEFT_UP(n))
				  for (x = 0; x < boxsize / 2 - RR(); x++)
				    for (y = 0; y < boxsize / 2 - RR(); y++)
				      THREEQUARTERSBIT(x, y);
				if (!IS_LEFT(n) && !IS_DOWN(n) && !IS_LEFT_DOWN(n))
				  for (x = 0; x < boxsize / 2 - RR(); x++)
				    for (y = boxsize / 2 + RR(); y < boxsize; y++)
				      THREEQUARTERSBIT(x, y);
				if (!IS_RIGHT(n) && !IS_UP(n) && !IS_RIGHT_UP(n))
				  for (x = boxsize / 2 + RR(); x < boxsize; x++)
				    for (y = 0; y < boxsize / 2 - RR(); y++)
				      THREEQUARTERSBIT(x, y);
				if (!IS_RIGHT(n) && !IS_DOWN(n) && !IS_RIGHT_DOWN(n))
				  for (x = boxsize / 2 + RR(); x < boxsize; x++)
				    for (y = boxsize / 2 + RR(); y < boxsize; y++)
				      THREEQUARTERSBIT(x, y);
			}
		} else if (LARGE_BELLYBUTTON) {
			if (!IS_LEFT(n) && !IS_UP(n) && !IS_LEFT_UP(n))
			  for (x=0;x<G(boxsize)+T(boxsize);x++)
			    for (y=0;y<G(boxsize)+T(boxsize);y++)
			      SETBIT(x, y);
			if (!IS_LEFT(n) && !IS_DOWN(n) && !IS_LEFT_DOWN(n))
			  for (x=0;x<G(boxsize)+T(boxsize);x++)
			    for (y=boxsize-G(boxsize)-T(boxsize);y<boxsize;y++)
			      SETBIT(x, y);
			if (!IS_RIGHT(n) && !IS_UP(n) && !IS_RIGHT_UP(n))
			  for (x=boxsize-G(boxsize)-T(boxsize);x<boxsize;x++)
			    for (y=0;y<G(boxsize)+T(boxsize);y++)
			      SETBIT(x, y);
			if (!IS_RIGHT(n) && !IS_DOWN(n) && !IS_RIGHT_DOWN(n))
			  for (x=boxsize-G(boxsize)-T(boxsize);x<boxsize;x++)
			    for (y=boxsize-G(boxsize)-T(boxsize);y<boxsize;y++)
			      SETBIT(x, y);
		}

		String[] stringData = new String[boxsize];

		for (y = 0; y < boxsize; y++)
			stringData[y] = new String(data[y]);
		//Pattern
		return PatternColorFilter.createImage(this, true,
			stringData, PATTERN_CHARS, patternColors);
	}

	void createBitmaps(boolean use3D, boolean drawNext,
			int boxsize, Color background) {
		int n = NUM_BITMAPS - 1;
		int plain = (use3D) ? 0 : 1;
		int next = (drawNext) ? 1 : 0;

		if ((pms[plain][next][n] = createBitmapPiece(n, use3D,
			boxsize, background)) == null)
		  return;
		for (n = 0; n < NUM_BITMAPS - 1; n++) {
			/* Avoid duplication of identical images. */
			if (IS_UP(n) && IS_DOWN(n) && IS_LEFT(n) && IS_RIGHT(n))
				pms[plain][next][n] = pms[plain][next][NUM_BITMAPS - 1];
			else if (IS_LEFT_UP(n) && (IS_LEFT(n) || IS_UP(n)))
				pms[plain][next][n] = pms[plain][next][n & ~LEFT_UP];
			else if (IS_LEFT_DOWN(n) && (IS_LEFT(n) || IS_DOWN(n)))
				pms[plain][next][n] = pms[plain][next][n & ~LEFT_DOWN];
			else if (IS_RIGHT_UP(n) && (IS_RIGHT(n) || IS_UP(n)))
				pms[plain][next][n] = pms[plain][next][n & ~RIGHT_UP];
			else if (IS_RIGHT_DOWN(n) && (IS_RIGHT(n) || IS_DOWN(n)))
				pms[plain][next][n] = pms[plain][next][n & ~RIGHT_DOWN];
			else if (imageNeeded(n) &&
					((pms[plain][next][n] = createBitmapPiece(n, use3D,
					boxsize, background)) == null)) {
				System.out.println("image needed but null: " + n);
				return;
			}
		}
	}

	static int ARR(int i, int j, int[][] shape, int size) {
		return ((i < 0 || i >= size || j < 0 || j >= size)
			? -2 : ((shape[j][i] > 0) ? 1 : 0));
	}

	static int polyPixmap(int i, int j, int[][] shape, int size) {
		int pmid = 0;

		if (ARR(i, j, shape, size) != ARR(i - 1, j, shape, size))
			pmid |= LEFT;
		if (ARR(i, j, shape, size) != ARR(i, j + 1, shape, size))
			pmid |= DOWN;
		if (ARR(i, j, shape, size) != ARR(i + 1, j, shape, size))
			pmid |= RIGHT;
		if (ARR(i, j, shape, size) != ARR(i, j - 1, shape, size))
			pmid |= UP;
		if (NUM_BITMAPS != SMALL) {
			if (ARR(i, j, shape, size) != ARR(i - 1, j + 1, shape, size))
				pmid |= LEFT_DOWN;
			if (ARR(i, j, shape, size) != ARR(i + 1, j + 1, shape, size))
				pmid |= RIGHT_DOWN;
			if (ARR(i, j, shape, size) != ARR(i + 1, j - 1, shape, size))
				pmid |= RIGHT_UP;
			if (ARR(i, j, shape, size) != ARR(i - 1, j - 1, shape, size))
				pmid |= LEFT_UP;
		}
		return pmid;
	}

	static void fillPolygon(Graphics g, int startX, int startY,
			int[] listX, int[] listY) {
		int n = listX.length;
		int[] tempListX = new int[n];
		int[] tempListY = new int[n];

		for (int i = 0; i < n; i++) {
			tempListX[i] = listX[i] + startX;
			tempListY[i] = listY[i] + startY;
		}
		g.fillPolygon(tempListX, tempListY, n);
	}

	static void drawPolygon(Graphics g, int startX, int startY,
			int[] listX, int[] listY) {
		int n = listX.length;
		int[] tempListX = new int[n];
		int[] tempListY = new int[n];

		for (int i = 0; i < n; i++) {
			tempListX[i] = listX[i] + startX;
			tempListY[i] = listY[i] + startY;
		}
		g.drawPolygon(tempListX, tempListY, n);
	}

	/* Pure colors can get brighter */
	static Color brighter(Color color) {
		int r = color.getRed();
		int g = color.getGreen();
		int b = color.getBlue();
		final double FACTOR = 0.8;

		/* From 2D group:
		 * 1. black.brighter() should return grey
		 * 2. applying brighter to blue will always return blue, brighter
		 * 3. non pure color (non zero rgb) will eventually return white		 */
												int i = (int) ((1 - FACTOR) * 255);

		if (r >= 0 && r < i)
			r = i;
		if (g >= 0 && g < i)
			g = i;
		if (b >= 0 && b < i)
			b = i;
		return new Color(Math.min((int) (r / FACTOR), 255),
			Math.min((int) (g / FACTOR), 255),
			Math.min((int) (b / FACTOR), 255));
	}

	void drawWallUnit(Graphics g,
			int pmid, Color cid, Color bid, int x, int y,
			int ulx, int uly, int base_x, int base_y,
			int lrx, int lry, int plateau_x, int plateau_y,
			int draw) {
		int MAX_SIDES = 4;

		if (g == null)
			return;
		int[] trapazoidListX = new int[MAX_SIDES];
		int[] trapazoidListY = new int[MAX_SIDES];

		/* if (color != background && color != xorColor) {
			XSetFillStyle(display, color, FillOpaqueStippled);
			XSetStipple(display, color, graypix);
		}*/
		trapazoidListX[0] = x;
		trapazoidListY[0] = y;
		trapazoidListX[1] = base_x;
		trapazoidListY[1] = base_y;
		trapazoidListX[2] = lrx - base_x - ulx;
		trapazoidListY[2] = lry - base_y - uly;
		trapazoidListX[3] = -plateau_x;
		trapazoidListY[3] = -plateau_y;
		/*trapazoidListX[4] = ulx + plateau_x - lrx;
		trapazoidListY[4] = ulx + plateau_y - lry;*/
		for (int k = 0; k < MAX_SIDES; k++) {
			trapazoidListX[k] +=
				((k > 0) ? trapazoidListX[k - 1] : ulx);
			trapazoidListY[k] +=
				((k > 0) ? trapazoidListY[k - 1] : uly);
		}
		g.setColor(cid);
		g.fillPolygon(trapazoidListX, trapazoidListY, MAX_SIDES);
		g.setColor(bid);
		g.drawPolygon(trapazoidListX, trapazoidListY, MAX_SIDES);
		/* Code added to use pmid info */
		if ((draw == DRAW_COLOR || draw == XOR_COLOR) &&
				!plainPiece) {
			int[] edgePointsX = new int[MAX_SIDES];
			int[] edgePointsY = new int[MAX_SIDES];
			int[] ptsX = new int[MAX_SIDES];
			int[] ptsY = new int[MAX_SIDES];
			int midPointX = 0, midPointY = 0;
			Color[] shade = new Color[MAX_SIDES];

			if (draw == XOR_COLOR) {
				if (pmid > NUM_BITMAPS) {
					shade[0] = brighter(cid);
					shade[3] = brighter(brighter(cid));
					shade[2] = cid.darker();
					shade[1] = cid.darker().darker();
				} else {
					shade[2] = brighter(cid);
					shade[3] = brighter(brighter(cid));
					shade[0] = cid.darker();
					shade[1] = cid.darker().darker();
				}
			} else {
				if (pmid > NUM_BITMAPS) {
					shade[2] = brighter(cid);
					shade[1] = brighter(brighter(cid));
					shade[0] = cid.darker();
					shade[3] = cid.darker().darker();
				} else {
					shade[0] = brighter(cid);
					shade[1] = brighter(brighter(cid));
					shade[2] = cid.darker();
					shade[3] = cid.darker().darker();
				}
			}
			for (int i = 0; i < MAX_SIDES; i++) {
				midPointX += trapazoidListX[i];
				midPointY += trapazoidListY[i];
			}
			midPointX /= MAX_SIDES;
			midPointY /= MAX_SIDES;
			for (int i = 0; i < MAX_SIDES - 1; i++) {
				edgePointsX[i] = (trapazoidListX[i + 1] +
					trapazoidListX[i]) >> 1;
				edgePointsY[i] = (trapazoidListY[i + 1] +
					trapazoidListY[i]) >> 1;
			}
			edgePointsX[MAX_SIDES - 1] = (trapazoidListX[0] +
				trapazoidListX[MAX_SIDES - 1]) >> 1;
			edgePointsY[MAX_SIDES - 1] = (trapazoidListY[0] +
				trapazoidListY[MAX_SIDES - 1]) >> 1;
			for (int i = 0; i < MAX_SIDES; i++) {
				int invi = (MAX_SIDES - i) % MAX_SIDES;
				int invi1 = (MAX_SIDES - i + 3) % MAX_SIDES;
				int invi3 = (MAX_SIDES - i + 1) % MAX_SIDES;
				int i1 = (i + 1) % MAX_SIDES;
				int i3 = (i + 3) % MAX_SIDES;

				if ((pmid & (1 << invi)) == 0) {
				    if ((pmid & (1 << invi3)) != 0 ||
						(pmid & (16 << invi)) != 0) {
					g.setColor(shade[i3]);
					ptsX[0] = trapazoidListX[i3];
					ptsY[0] = trapazoidListY[i3];
					ptsX[1] = midPointX;
					ptsY[1] = midPointY;
					ptsX[2] = edgePointsX[i3];
					ptsY[2] = edgePointsY[i3];
					ptsX[3] = trapazoidListX[i3];
					ptsY[3] = trapazoidListY[i3];
					g.fillPolygon(ptsX, ptsY, MAX_SIDES);
					g.drawPolygon(ptsX, ptsY, MAX_SIDES);
				    }
				    if ((pmid & (1 << invi1)) != 0 ||
						(pmid & (16 << invi1)) != 0) {
					g.setColor(shade[i1]);
					ptsX[0] = trapazoidListX[i];
					ptsY[0] = trapazoidListY[i];
					ptsX[1] = midPointX;
					ptsY[1] = midPointY;
					ptsX[2] = edgePointsX[i3];
					ptsY[2] = edgePointsY[i3];
					ptsX[3] = trapazoidListX[i];
					ptsY[3] = trapazoidListY[i];
					g.fillPolygon(ptsX, ptsY, MAX_SIDES);
					g.drawPolygon(ptsX, ptsY, MAX_SIDES);
				    }
				} else {
					g.setColor(shade[i]);
					ptsX[0] = trapazoidListX[i3];
					ptsY[0] = trapazoidListY[i3];
					ptsX[1] = trapazoidListX[i];
					ptsY[1] = trapazoidListY[i];
					ptsX[2] = midPointX;
					ptsY[2] = midPointY;
					ptsX[3] = trapazoidListX[i3];
					ptsY[3] = trapazoidListY[i3];
					g.fillPolygon(ptsX, ptsY, MAX_SIDES);
					g.drawPolygon(ptsX, ptsY, MAX_SIDES);
				}
			}
		}
	}

	static void drawWallUnitBorder(Graphics g, int x, int y,
			int ulx, int uly, int base_x, int base_y,
			int lrx, int lry, int plateau_x, int plateau_y) {
		if (g == null)
			return;
		int[] trapazoidListX = new int[4];
		int[] trapazoidListY = new int[4];

		trapazoidListX[0] = x;
		trapazoidListY[0] = y;
		trapazoidListX[1] = base_x;
		trapazoidListY[1] = base_y;
		trapazoidListX[2] = lrx - base_x - ulx;
		trapazoidListY[2] = lry - base_y - uly;
		trapazoidListX[3] = -plateau_x;
		trapazoidListY[3] = -plateau_y;
		/*trapazoidListX[4] = ulx + plateau_x - lrx;
		trapazoidListY[4] = ulx + plateau_y - lry;*/
		for (int k = 0; k < 4; k++) {
			trapazoidListX[k] +=
				((k > 0) ? trapazoidListX[k - 1] : 0);
			trapazoidListY[k] +=
				((k > 0) ? trapazoidListY[k - 1] : 0);
		}
		drawPolygon(g, ulx, uly, trapazoidListX, trapazoidListY);
	}

	void drawUnit(Graphics g, boolean drawNext, int pmid, Color cid,
			int x, int y, int i, int j, int width, int height) {
		int plain = (plainPiece) ? 1 : 0;
		int next = (drawNext) ? 1 : 0;

		if (g == null)
			return;
		g.setColor(cid);
		g.fillRect(x + width * i, y + height * j, width, height);
		g.drawImage(pms[plain][next][pmid % NUM_BITMAPS],
			x + width * i, y + height * j, width, height, null);
	}

	void drawUnit(Graphics g, boolean drawNext, int pmid, int cid,
			int x, int y, int i, int j, int width, int height) {
		drawUnit(g, drawNext, pmid, colorThings[cid],
			x, y, i, j, width, height);
	}

	public static void clearUnits(Graphics g, Color color,
			 int x, int y, int i, int j, int width, int height) {
		if (g == null)
			return;
		g.setColor(color);
		g.fillRect(x + width * i, y + height * j, width, height);
	}

	void setPlainPiece(boolean plainPiece) {
		this.plainPiece = plainPiece;
	}

	boolean getPlainPiece() {
		return this.plainPiece;
	}
}
